import java.io.*;
import java.security.*;
import java.util.*;
import java.util.List;

public class AbstractWars {
    SecureRandom rnd;
    int B;
    int[] baseX, baseY;
    AbstractWars() {
      try {
        rnd = SecureRandom.getInstance("SHA1PRNG");
      } catch (Exception e) { }
        rnd.setSeed(123);
    }
    // ----------------------------------------------
    public int init(int[] baseLocations, int speed) { 
        B = baseLocations.length / 2;
        baseX = new int[B];
        baseY = new int[B];
        for (int i = 0; i < B; ++i) {
            baseX[i] = baseLocations[2*i];
            baseY[i] = baseLocations[2*i+1];
        }
        return 0;
    }
    // ----------------------------------------------
    ArrayList<Integer> others;
    // picks a random base to attack based on distance to the opponent bases: the closer the base, the higher the chances are
    public int getRandomBase(int sourceInd) {
        double[] probs = new double[others.size()];
        double sp = 0;
        for (int i = 0; i < others.size(); ++i) {
            int ind = others.get(i).intValue();
            probs[i] = 1 / (Math.pow(baseX[sourceInd] - baseX[ind], 2) + Math.pow(baseY[sourceInd] - baseY[ind], 2));
            sp += probs[i];
        }
        
        double r = rnd.nextDouble() * sp;
        double s = 0;
        for (int i = 0; i < others.size(); ++i) {
            s += probs[i];
            if (s >= r)
                return others.get(i).intValue();
        }
        return others.get(others.size() - 1).intValue();
    }
    // ----------------------------------------------
    int[] sendTroops(int[] bases, int[] troops) {
        // compile the list of bases owned by other players
        others = new ArrayList<Integer>();
        for (int i = 0; i < B; ++i)
            if (bases[2*i] != 0)
                others.add(i);
        if (others.size() == 0) {
            // noone to fight!
            return new int[0];
        }

        ArrayList<Integer> att = new ArrayList<Integer>();
        for (int i = 0; i < B; ++i) {
            if (bases[2*i] == 0 && bases[2*i+1] > 1000 * 2 / 3) {
                // send troops to a random base of different ownership
                att.add(i);
                att.add(getRandomBase(i));
            }
        }
        int[] ret = new int[att.size()];
        for (int i = 0; i < att.size(); ++i)
            ret[i] = att.get(i).intValue();
        return ret;
    }
    // -------8<------- end of solution submitted to the website -------8<-------
    public static void main(String[] args) {
    try {
        AbstractWars aw = new AbstractWars();

        BufferedReader br = new BufferedReader(new InputStreamReader(System.in));

        int N = Integer.parseInt(br.readLine());
        int[] baseLoc = new int[N];
        for (int i = 0; i < N; ++i) {
            baseLoc[i] = Integer.parseInt(br.readLine());
        }
        int S = Integer.parseInt(br.readLine());
        int retInit = aw.init(baseLoc, S);
        System.out.println(retInit);
        System.out.flush();

        for (int st = 0; st < 2000; ++st) {
            int B = Integer.parseInt(br.readLine());
            int[] bases = new int[B];
            for (int i = 0; i < B; ++i)
                bases[i] = Integer.parseInt(br.readLine());
            int T = Integer.parseInt(br.readLine());
            int[] troops = new int[T];
            for (int i = 0; i < T; ++i)
                troops[i] = Integer.parseInt(br.readLine());

            int[] ret = aw.sendTroops(bases, troops);
            System.out.println(ret.length);
            for (int i = 0; i < ret.length; ++i) {
                System.out.println(ret[i]);
            }
            System.out.flush();
        }
    }
    catch (Exception e) {}
    }
}
